// ContainView.cpp : implementation of the CContainView class
//

#include "stdafx.h"
#include "OContain.h"

#include "ContainDoc.h"
#include "CntrItem.h"
#include "ContainView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CContainView

IMPLEMENT_DYNCREATE(CContainView, CView)

BEGIN_MESSAGE_MAP(CContainView, CView)
	//{{AFX_MSG_MAP(CContainView)
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
	ON_COMMAND(ID_OLE_INSERT_NEW, OnInsertObject)
	ON_COMMAND(ID_CANCEL_EDIT_CNTR, OnCancelEditCntr)
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_SETCURSOR()
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CContainView construction/destruction

CContainView::CContainView()
{
	m_pSelection = NULL;
	// TODO: add construction code here

}

CContainView::~CContainView()
{
}

BOOL CContainView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CContainView drawing

void CContainView::OnDraw(CDC* pDC)
{
	CContainDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	POSITION pos = pDoc->GetStartPosition();
	while (pos != NULL)
	{
	   // draw the item
	   m_pSelection = (CContainCntrItem*)pDoc->GetNextItem(pos);
	   m_pSelection->Draw(pDC, m_pSelection->m_rect);

	   // Draw tracker rectangle over the item
	   CRectTracker tracker;
	   SetupTracker(m_pSelection, &tracker);
	   tracker.Draw(pDC);
	}
}

void CContainView::OnInitialUpdate()
{
	CView::OnInitialUpdate();

	// TODO: remove this code when final selection model code is written
	m_pSelection = NULL;    // initialize selection

}

/////////////////////////////////////////////////////////////////////////////
// OLE Client support and commands

BOOL CContainView::IsSelected(const CObject* pDocItem) const
{
	// The implementation below is adequate if your selection consists of
	//  only CContainCntrItem objects.  To handle different selection
	//  mechanisms, the implementation here should be replaced.

	// TODO: implement this function that tests for a selected OLE client item

	return pDocItem == m_pSelection;
}

void CContainView::OnInsertObject()
{
	// Invoke the standard Insert Object dialog box to obtain information
	//  for new CContainCntrItem object.
	COleInsertDialog dlg;
	if (dlg.DoModal() != IDOK)
		return;

	BeginWaitCursor();

	CContainCntrItem* pItem = NULL;
	TRY
	{
		// Create new item connected to this document.
		CContainDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		pItem = new CContainCntrItem(pDoc);
		ASSERT_VALID(pItem);

		// Initialize the item from the dialog data.
		if (!dlg.CreateItem(pItem))
			AfxThrowMemoryException();  // any exception will do
		ASSERT_VALID(pItem);

		// If item created from class list (not from file) then launch
		//  the server to edit the item.
		if (dlg.GetSelectionType() == COleInsertDialog::createNewItem)
			pItem->DoVerb(OLEIVERB_SHOW, this);

		ASSERT_VALID(pItem);

		// As an arbitrary user interface design, this sets the selection
		//  to the last item inserted.

		// TODO: reimplement selection as appropriate for your application

		m_pSelection = pItem;   // set selection to last inserted item
		pDoc->UpdateAllViews(NULL);
	}
	CATCH(CException, e)
	{
		if (pItem != NULL)
		{
			ASSERT_VALID(pItem);
			pItem->Delete();
		}
		AfxMessageBox(IDP_FAILED_TO_CREATE);
	}
	END_CATCH

	EndWaitCursor();
}

// The following command handler provides the standard keyboard
//  user interface to cancel an in-place editing session.  Here,
//  the container (not the server) causes the deactivation.
void CContainView::OnCancelEditCntr()
{
	// Close any in-place active item on this view.
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
	{
		pActiveItem->Close();
	}
	ASSERT(GetDocument()->GetInPlaceActiveItem(this) == NULL);
}

// Special handling of OnSetFocus and OnSize are required for a container
//  when an object is being edited in-place.
void CContainView::OnSetFocus(CWnd* pOldWnd)
{
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL &&
		pActiveItem->GetItemState() == COleClientItem::activeUIState)
	{
		// need to set focus to this item if it is in the same view
		CWnd* pWnd = pActiveItem->GetInPlaceWindow();
		if (pWnd != NULL)
		{
			pWnd->SetFocus();   // don't call the base class
			return;
		}
	}

	CView::OnSetFocus(pOldWnd);
}

void CContainView::OnSize(UINT nType, int cx, int cy)
{
	CView::OnSize(nType, cx, cy);
	COleClientItem* pActiveItem = GetDocument()->GetInPlaceActiveItem(this);
	if (pActiveItem != NULL)
		pActiveItem->SetItemRects();
}


CContainCntrItem* CContainView::HitTestItems(CPoint point)
{
	CContainDoc* pDoc = GetDocument();
	CContainCntrItem* pItemHit = NULL;
	POSITION pos = pDoc->GetStartPosition();
	while (pos != NULL)
	{
	     CContainCntrItem* pItem = (CContainCntrItem*)pDoc->GetNextItem(pos);
		 if (pItem->m_rect.PtInRect(point))
		    pItemHit = pItem;
	}
	
	// Return top item at point
	return pItemHit;

}


void CContainView::SetSelection(CContainCntrItem* pItem)
{
   // close in-place active item
   if (pItem == NULL || m_pSelection != pItem)
   {
      COleClientItem* pActiveItem 
        = GetDocument()->GetInPlaceActiveItem(this);
	  if (pActiveItem != NULL && pActiveItem != pItem)
	    pActiveItem->Close();
   }

   // update view to new selection
   if (m_pSelection != pItem)
   {
      if (m_pSelection != NULL)
	    OnUpdate(NULL, HINT_UPDATE_ITEM, m_pSelection);

	m_pSelection = pItem;
	if (m_pSelection != NULL)
	   OnUpdate(NULL, HINT_UPDATE_ITEM, m_pSelection);

   }
}


void CContainView::SetupTracker(CContainCntrItem* pItem, 
                               CRectTracker* pTracker)
{
	// Setup styles of tracker rectangle

    pTracker->m_rect = pItem->m_rect;

	if (pItem == m_pSelection)
	   pTracker->m_nStyle |= CRectTracker::resizeInside;

	if (pItem->GetType() == OT_LINK)
	   pTracker->m_nStyle |= CRectTracker::dottedLine;
	else
	   pTracker->m_nStyle |= CRectTracker::solidLine;

	if (pItem->GetItemState() == COleClientItem::openState ||
	   pItem->GetItemState() == COleClientItem::activeUIState)
	{
	   pTracker->m_nStyle |= CRectTracker::hatchInside;
	}


}

/////////////////////////////////////////////////////////////////////////////
// CContainView diagnostics

#ifdef _DEBUG
void CContainView::AssertValid() const
{
	CView::AssertValid();
}

void CContainView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CContainDoc* CContainView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CContainDoc)));
	return (CContainDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CContainView message handlers

void CContainView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags, point);
	
	if (m_pSelection != NULL)
	{
	   m_pSelection->DoVerb(GetKeyState(VK_CONTROL) < 0 ?
	      OLEIVERB_OPEN : OLEIVERB_PRIMARY, this);
	}
	
	// default processing	
	CView::OnLButtonDblClk(nFlags, point);
}



void CContainView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CContainCntrItem* pItemHit = HitTestItems(point);
	SetSelection(pItemHit);
	
	if (pItemHit != NULL)
	{
	   CRectTracker tracker;
	   SetupTracker(pItemHit, &tracker);
	   
	   UpdateWindow();
	   if(tracker.Track(this, point))
	   {
		  pItemHit->InvalidateItem();
		  pItemHit->m_rect = tracker.m_rect;
		  pItemHit->InvalidateItem();

	      GetDocument()->SetModifiedFlag();
	   }
	}

	// Default processing    	
	CView::OnLButtonDown(nFlags, point);
}

BOOL CContainView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if (pWnd == this && m_pSelection != NULL)
	{
	   // give the tracker for the selection a change 
	   CRectTracker tracker;
	   SetupTracker(m_pSelection, &tracker);
	   if (tracker.SetCursor(this, nHitTest))
	      return TRUE;
	}
	      
	// otherwise, default processing	
	return CView::OnSetCursor(pWnd, nHitTest, message);
}



void CContainView::OnEditClear() 
{
	if (m_pSelection != NULL)
	{
	   m_pSelection->Delete();
	   m_pSelection = NULL;
	   GetDocument()->UpdateAllViews(NULL);
	}	
}

void CContainView::OnEditCopy() 
{
	// Implement the Copy Command on the Edit Menu
	if (m_pSelection != NULL)
	   m_pSelection->CopyToClipboard();	
}


void CContainView::OnEditPaste() 
{
	CContainCntrItem* pItem = NULL;
	
	TRY
	{
	   // Create new item connected to this document.
	   CContainDoc* pDoc = GetDocument();
	   ASSERT_VALID(pDoc);

	   pItem = new CContainCntrItem(pDoc);	
	   ASSERT_VALID(pItem);

	   // Initialize the item from clipboard data
	   if (!pItem->CreateFromClipboard())
	      AfxThrowMemoryException();
	   ASSERT_VALID(pItem);
	   
	   // update the size before displaying
	   pItem->UpdateFromServerExtent();
	   
	   // set selection to newly pasted item
	   SetSelection(pItem);
	   pItem->InvalidateItem();
	} 
	CATCH(CException, e)
	{
	   if (pItem != NULL)
	   {
	      ASSERT_VALID(pItem);
		  pItem->Delete();
	   }
	   AfxMessageBox(IDP_FAILED_TO_CREATE);
	}
	END_CATCH
}

void CContainView::OnUpdateEditClear(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSelection != NULL);
}


void CContainView::OnUpdateEditCopy(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_pSelection != NULL);	
}

